% Evaluation code for image segmentation
%
% Your goal is to implement color and texture based segmentation algorithms
%
% This code is part of:
%
%   CMPSCI 670: Computer Vision, Fall 2014
%   University of Massachusetts, Amherst
%   Instructor: Subhransu Maji
%
%   Homework 4: Grouping and Texture

imageName = 'zebra.jpg'; 
dataDir = fullfile('..','data');
outputDir = fullfile('..','output'); % This is where your output will be written
im = imread(fullfile(dataDir, imageName));
im = im2double(im);
clear param;

%% compute filterBank
fb = makeFilterbank();
showFilterbank(fb);
print(gcf, fullfile(outputDir, 'fb.png'),'-dpng');


%% Simple color quanatization
param.feature = 'rgb';
param.k = 5; % Set various values here k=2, 5, 10
segColor = segmentImage(im, param);
colorTitle=sprintf('k-means (k=%i), feature: rgb', param.k);

% Texton histogram clustering
param.feature = 'texton';
param.numTextons = 20;
param.textonHistRadius = 5;
param.k = 5; % Set various values here k=2, 5, 10
segTexton = segmentImage(im, param);
textonTitle = sprintf('k-means (k=%i), feature: texton', param.k);

%% Display outputs
figure; clf;
subplot(1,3,1);
imagesc(im); axis image off;
title('Original image');

subplot(1,3,2);
imagesc(segColor); axis image off; colormap jet;
title(colorTitle);

subplot(1,3,3);
imagesc(segTexton); axis image off; colormap jet;
title(textonTitle);

% Save output
print(gcf, fullfile(outputDir, 'segmentation.png'),'-dpng');