% Evaluation code for texture classification
%
% Your goal is to implement color and texture based segmentation algorithms
%
% This code is part of:
%
%   CMPSCI 670: Computer Vision, Fall 2014
%   University of Massachusetts, Amherst
%   Instructor: Subhransu Maji
%
%   Homework 4: Grouping and Texture

% Set this to the location of your data directory
dataDir = fullfile('..','data', 'uiuc_texture');

% Load all the image names
files = dir(fullfile(dataDir, '*.jpg'));
data.image = cell(length(files),1);
data.class = cell(length(files),1);
for i = 1:length(data.class)
    data.image{i} = files(i).name;
    ind = strfind(data.image{i}, '_');
    data.class{i} = data.image{i}(1:ind(1)-1);
end
[~,~,data.classId] = unique(data.class);


%% Make filterBank
fb = makeFilterbank();

%% Loop over images and compute histograms of response
numImages = length(data.class);
numFilters = size(fb,3);
fbHist = zeros(numImages, numFilters);
fprintf('[fbHist %i images]', numImages);
for i = 1:numImages, 
    im = imread(fullfile(dataDir, data.image{i}));
    fbHist(i,:) = computeFbHist(im, fb);
    fprintf('.');
end
fprintf('[done]\n');

%% Evaluate leave-one-out error
% For each image find the most similar image (excluding itself) in the
% dataset and copy the class label from it. 

trueLabel = data.classId;
predLabel = zeros(numImages,1);
for i = 1:numImages, 
    d = fbHist - repmat(fbHist(i,:),numImages,1);
    distance = sum(d.^2,2);
    distance(i) = inf;
    [~,ind] = min(distance);
    predLabel(i) = trueLabel(ind);
end
accuracy = mean(trueLabel == predLabel);
fprintf('Classification accuracy: %.2f%%\n', accuracy*100);

%% Show top 10 images for a subset of images
numClass = length(unique(data.classId));
numDisplay = 10;
plotId = 1;
figure;
for c = 1:numClass, 
    i = find(data.classId==c, 1); % Find the first image of this class
    d = fbHist - repmat(fbHist(i,:),numImages,1);
    distance = sum(d.^2,2);
    [~, ord] = sort(distance);
    for d = 1:numDisplay
        
        % Load the image
        im = imread(fullfile(dataDir, data.image{ord(d)}));
        
        % Plot this
        subplot(numClass, numDisplay, plotId);
        imshow(im); axis image off;
        title(data.class{ord(d)});
        plotId = plotId + 1;
    end
end

% Save output
print(gcf, fullfile(outputDir, 'top10.png'),'-dpng');


%% Visualize classes points in 2D using PCA
COEFF = princomp(fbHist);
points = fbHist*COEFF;
points = points(:,1:2); %Only use the first two coordinates

pointColors = 'rgbmk';

figure;
hold on;

clear legendText;
for c = 1:numClass,
    i = find(data.classId==c,1);
    classId = data.classId(i);
    plot(points(i,1), points(i,2), [pointColors(classId) '.'], 'MarkerSize',20);
    legendText{c} = data.class{i};
end

for i = 1:numImages, 
    classId = data.classId(i);
    plot(points(i,1), points(i,2), [pointColors(classId) '.'], 'MarkerSize',20);
end

legend(legendText);
axis equal; box on;
xlabel('PCA component 1');
ylabel('PCA component 2');
title('Texture classes PCA histogram');

% Save output
print(gcf, fullfile(outputDir, 'texturePCA.png'),'-dpng');




